@@ -1,5 +1,5 @@
 # -*- test-case-name: twisted.test.test_defer,twisted.test.test_defgen,twisted.internet.test.test_inlinecb -*-
-# Copyright (c) 2001-2010 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -17,6 +17,7 @@ Maintainer: Glyph Lefkowitz
 """
 
 import traceback
+import types
 import warnings
 from sys import exc_info
 
@@ -132,7 +133,7 @@ def maybeDeferred(f, *args, **kw):
     try:
         result = f(*args, **kw)
     except:
-        return fail(failure.Failure())
+        return fail(failure.Failure(captureVars=Deferred.debug))
 
     if isinstance(result, Deferred):
         return result
@@ -190,7 +191,7 @@ class Deferred:
     that come from outside packages that are not under our control, we use
     threads (see for example L{twisted.enterprise.adbapi}).
 
-    For more information about Deferreds, see doc/howto/core/defer.html or
+    For more information about Deferreds, see doc/core/howto/defer.html or
     U{http://twistedmatrix.com/documents/current/core/howto/defer.html}
 
     When creating a Deferred, you may provide a canceller function, which
@@ -275,6 +276,9 @@ class Deferred:
         Add a pair of callbacks (success and error) to this L{Deferred}.
 
         These will be executed when the 'master' callback is run.
+
+        @return: C{self}.
+        @rtype: a L{Deferred}
         """
         assert callable(callback)
         assert errback == None or callable(errback)
@@ -340,6 +344,9 @@ class Deferred:
         being returned from a callback, it is possible to cause the call
         stack size limit to be exceeded by chaining many L{Deferred}s
         together with C{chainDeferred}.
+
+        @return: C{self}.
+        @rtype: a L{Deferred}
         """
         d._chainedTo = self
         return self.addCallbacks(d.callback, d.errback)
@@ -382,7 +389,9 @@ class Deferred:
         @raise NoCurrentExceptionError: If C{fail} is C{None} but there is
             no current exception state.
         """
-        if not isinstance(fail, failure.Failure):
+        if fail is None:
+            fail = failure.Failure(captureVars=self.debug)
+        elif not isinstance(fail, failure.Failure):
             fail = failure.Failure(fail)
 
         self._startRunCallbacks(fail)
@@ -543,7 +552,9 @@ class Deferred:
                     finally:
                         current._runningCallbacks = False
                 except:
-                    current.result = failure.Failure()
+                    # Including full frame information in the Failure is quite
+                    # expensive, so we avoid it unless self.debug is set.
+                    current.result = failure.Failure(captureVars=self.debug)
                 else:
                     if isinstance(current.result, Deferred):
                         # The result is another Deferred.  If it has a result,
@@ -708,7 +719,7 @@ class DeferredList(Deferred):
     L{DeferredList} is implemented by adding callbacks and errbacks to each
     L{Deferred} in the list passed to it.  This means callbacks and errbacks
     added to the Deferreds before they are passed to L{DeferredList} will change
-    the result that L{DeferredList} sees (ie, L{DeferredList} is not special).
+    the result that L{DeferredList} sees (i.e., L{DeferredList} is not special).
     Callbacks and errbacks can also be added to the Deferreds after they are
     passed to L{DeferredList} and L{DeferredList} may change the result that
     they see.
@@ -731,9 +742,9 @@ class DeferredList(Deferred):
             L{DeferredList} will fire when the first L{Deferred} in
             C{deferredList} fires with a non-failure result without waiting for
             any of the other Deferreds.  When this flag is set, the DeferredList
-            will fire with a two-tuple: the first element is the index in
-            C{deferredList} of the Deferred which fired; the second element is
-            the result of that Deferred.
+            will fire with a two-tuple: the first element is the result of the
+            Deferred which fired; the second element is the index in
+            C{deferredList} of that Deferred.
         @type fireOnOneCallback: C{bool}
 
         @param fireOnOneErrback: (keyword param) a flag indicating that this
@@ -805,16 +816,30 @@ def _parseDListResult(l, fireOnOneErrback=False):
 
 
 
-def gatherResults(deferredList):
+def gatherResults(deferredList, consumeErrors=False):
     """
-    Returns list with result of given L{Deferred}s.
+    Returns, via a L{Deferred}, a list with the results of the given
+    L{Deferred}s - in effect, a "join" of multiple deferred operations.
 
-    This builds on L{DeferredList} but is useful since you don't
-    need to parse the result for success/failure.
+    The returned L{Deferred} will fire when I{all} of the provided L{Deferred}s
+    have fired, or when any one of them has failed.
+
+    This differs from L{DeferredList} in that you don't need to parse
+    the result for success/failure.
 
     @type deferredList:  C{list} of L{Deferred}s
+
+    @param consumeErrors: (keyword param) a flag, defaulting to False,
+        indicating that failures in any of the given L{Deferreds} should not be
+        propagated to errbacks added to the individual L{Deferreds} after this
+        L{gatherResults} invocation.  Any such errors in the individual
+        L{Deferred}s will be converted to a callback result of C{None}.  This
+        is useful to prevent spurious 'Unhandled error in Deferred' messages
+        from being logged.  This parameter is available since 11.1.0.
+    @type consumeErrors: C{bool}
     """
-    d = DeferredList(deferredList, fireOnOneErrback=True)
+    d = DeferredList(deferredList, fireOnOneErrback=True,
+                                   consumeErrors=consumeErrors)
     d.addCallback(_parseDListResult)
     return d
 
@@ -1117,6 +1142,10 @@ def inlineCallbacks(f):
     available. The generator will be sent the result of the L{Deferred} with the
     'send' method on generators, or if the result was a failure, 'throw'.
 
+    Things that are not L{Deferred}s may also be yielded, and your generator
+    will be resumed with the same object sent back. This means C{yield}
+    performs an operation roughly equivalent to L{maybeDeferred}.
+
     Your inlineCallbacks-enabled generator will return a L{Deferred} object, which
     will result in the return value of the generator (or will fail with a
     failure object if your generator raises an unhandled exception). Note that
@@ -1124,6 +1153,13 @@ def inlineCallbacks(f):
     instead. Falling off the end of the generator, or simply using C{return}
     will cause the L{Deferred} to have a result of C{None}.
 
+    Be aware that L{returnValue} will not accept a L{Deferred} as a parameter.
+    If you believe the thing you'd like to return could be a L{Deferred}, do
+    this::
+
+        result = yield result
+        returnValue(result)
+
     The L{Deferred} returned from your deferred generator may errback if your
     generator raised an exception::
 
@@ -1138,7 +1174,17 @@ def inlineCallbacks(f):
                 raise Exception('DESTROY ALL LIFE')
     """
     def unwindGenerator(*args, **kwargs):
-        return _inlineCallbacks(None, f(*args, **kwargs), Deferred())
+        try:
+            gen = f(*args, **kwargs)
+        except _DefGen_Return:
+            raise TypeError(
+                "inlineCallbacks requires %r to produce a generator; instead"
+                "caught returnValue being used in a non-generator" % (f,))
+        if not isinstance(gen, types.GeneratorType):
+            raise TypeError(
+                "inlineCallbacks requires %r to produce a generator; "
+                "instead got %r" % (f, gen))
+        return _inlineCallbacks(None, gen, Deferred())
     return mergeFunctionMetadata(f, unwindGenerator)
 
 
