@@ -7,11 +7,12 @@ TCP support for IOCP reactor
 
 import socket, operator, errno, struct
 
-from zope.interface import implements, directlyProvides
+from zope.interface import implements, classImplements
 
 from twisted.internet import interfaces, error, address, main, defer
-from twisted.internet.abstract import isIPAddress
+from twisted.internet.abstract import _LogOwner, isIPAddress, isIPv6Address
 from twisted.internet.tcp import _SocketCloser, Connector as TCPConnector
+from twisted.internet.tcp import _AbortingMixin
 from twisted.python import log, failure, reflect, util
 
 from twisted.internet.iocpreactor import iocpsupport as _iocp, abstract
@@ -23,12 +24,9 @@ from twisted.internet.iocpreactor.const import ERROR_CONNECTION_REFUSED
 from twisted.internet.iocpreactor.const import ERROR_NETWORK_UNREACHABLE
 
 try:
-    from twisted.protocols.tls import TLSMemoryBIOFactory, TLSMemoryBIOProtocol
+    from twisted.internet._newtls import startTLS as _startTLS
 except ImportError:
-    TLSMemoryBIOProtocol = TLSMemoryBIOFactory = None
-    _extraInterfaces = ()
-else:
-    _extraInterfaces = (interfaces.ITLSTransport,)
+    _startTLS = None
 
 # ConnectEx returns these. XXX: find out what it does for timeout
 connectExErrors = {
@@ -36,55 +34,17 @@ connectExErrors = {
         ERROR_NETWORK_UNREACHABLE: errno.WSAENETUNREACH,
         }
 
-
-class _BypassTLS(object):
+class Connection(abstract.FileHandle, _SocketCloser, _AbortingMixin):
     """
-    L{_BypassTLS} is used as the transport object for the TLS protocol object
-    used to implement C{startTLS}.  Its methods skip any TLS logic which
-    C{startTLS} enables.
-
-    @ivar _connection: A L{Connection} which TLS has been started on which will
-        be proxied to by this object.  Any method which has its behavior
-        altered after C{startTLS} will be skipped in favor of the base class's
-        implementation.  This allows the TLS protocol object to have direct
-        access to the transport, necessary to actually implement TLS.
-    """
-    def __init__(self, connection):
-        self._connection = connection
-
-
-    def __getattr__(self, name):
-        return getattr(self._connection, name)
-
-
-    def write(self, data):
-        return abstract.FileHandle.write(self._connection, data)
-
-
-    def writeSequence(self, iovec):
-        return abstract.FileHandle.writeSequence(self._connection, iovec)
-
-
-    def loseConnection(self, reason=None):
-        return abstract.FileHandle.loseConnection(self._connection, reason)
-
-
-
-class Connection(abstract.FileHandle, _SocketCloser):
-    """
-    @ivar _tls: C{False} to indicate the connection is in normal TCP mode,
+    @ivar TLS: C{False} to indicate the connection is in normal TCP mode,
         C{True} to indicate that TLS has been started and that operations must
         be routed through the L{TLSMemoryBIOProtocol} instance.
-
-    @ivar _tlsClientDefault: A flag which must be set by a subclass.  If set to
-        C{True}, L{startTLS} will default to initiating SSL as a client.  If
-        set to C{False}, L{startTLS} will default to initiating SSL as a
-        server.
     """
     implements(IReadWriteHandle, interfaces.ITCPTransport,
-               interfaces.ISystemHandle, *_extraInterfaces)
+               interfaces.ISystemHandle)
+
+    TLS = False
 
-    _tls = False
 
     def __init__(self, sock, proto, reactor=None):
         abstract.FileHandle.__init__(self, reactor)
@@ -143,8 +103,12 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
 
     def connectionLost(self, reason):
+        if self.disconnected:
+            return
         abstract.FileHandle.connectionLost(self, reason)
-        self._closeSocket()
+        isClean = (reason is None or
+                   not reason.check(error.ConnectionAborted))
+        self._closeSocket(isClean)
         protocol = self.protocol
         del self.protocol
         del self.socket
@@ -177,36 +141,12 @@ class Connection(abstract.FileHandle, _SocketCloser):
         self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, enabled)
 
 
-    if TLSMemoryBIOFactory is not None:
+    if _startTLS is not None:
         def startTLS(self, contextFactory, normal=True):
             """
             @see: L{ITLSTransport.startTLS}
             """
-            # Figure out which direction the SSL goes in.  If normal is True,
-            # we'll go in the direction indicated by the subclass.  Otherwise,
-            # we'll go the other way (client = not normal ^ _tlsClientDefault,
-            # in other words).
-            if normal:
-                client = self._tlsClientDefault
-            else:
-                client = not self._tlsClientDefault
-
-            tlsFactory = TLSMemoryBIOFactory(contextFactory, client, None)
-            tlsProtocol = TLSMemoryBIOProtocol(tlsFactory, self.protocol, False)
-            self.protocol = tlsProtocol
-
-            self.getHandle = tlsProtocol.getHandle
-            self.getPeerCertificate = tlsProtocol.getPeerCertificate
-
-            # Mark the transport as secure.
-            directlyProvides(self, interfaces.ISSLTransport)
-
-            # Remember we did this so that write and writeSequence can send the
-            # data to the right place.
-            self._tls = True
-
-            # Hook it up
-            self.protocol.makeConnection(_BypassTLS(self))
+            _startTLS(self, contextFactory, normal, abstract.FileHandle)
 
 
     def write(self, data):
@@ -217,7 +157,9 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.write}
         """
-        if self._tls:
+        if self.disconnected:
+            return
+        if self.TLS:
             self.protocol.write(data)
         else:
             abstract.FileHandle.write(self, data)
@@ -231,7 +173,9 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.writeSequence}
         """
-        if self._tls:
+        if self.disconnected:
+            return
+        if self.TLS:
             self.protocol.writeSequence(iovec)
         else:
             abstract.FileHandle.writeSequence(self, iovec)
@@ -244,15 +188,51 @@ class Connection(abstract.FileHandle, _SocketCloser):
 
         @see: L{ITCPTransport.loseConnection}
         """
-        if self._tls:
+        if self.TLS:
             if self.connected and not self.disconnecting:
                 self.protocol.loseConnection()
         else:
             abstract.FileHandle.loseConnection(self, reason)
 
 
+    def registerProducer(self, producer, streaming):
+        """
+        Register a producer.
+
+        If TLS is enabled, the TLS connection handles this.
+        """
+        if self.TLS:
+            # Registering a producer before we're connected shouldn't be a
+            # problem. If we end up with a write(), that's already handled in
+            # the write() code above, and there are no other potential
+            # side-effects.
+            self.protocol.registerProducer(producer, streaming)
+        else:
+            abstract.FileHandle.registerProducer(self, producer, streaming)
+
+
+    def unregisterProducer(self):
+        """
+        Unregister a producer.
+
+        If TLS is enabled, the TLS connection handles this.
+        """
+        if self.TLS:
+            self.protocol.unregisterProducer()
+        else:
+            abstract.FileHandle.unregisterProducer(self)
+
+if _startTLS is not None:
+    classImplements(Connection, interfaces.ITLSTransport)
+
+
 
 class Client(Connection):
+    """
+    @ivar _tlsClientDefault: Always C{True}, indicating that this is a client
+        connection, and by default when TLS is negotiated this class will act as
+        a TLS client.
+    """
     addressFamily = socket.AF_INET
     socketType = socket.SOCK_STREAM
 
@@ -302,7 +282,7 @@ class Client(Connection):
             return
 
         try:
-            self._closeSocket()
+            self._closeSocket(True)
         except AttributeError:
             pass
         else:
@@ -326,12 +306,13 @@ class Client(Connection):
             self.failIfNotConnected(error.getConnectError((rc,
                                     errno.errorcode.get(rc, 'Unknown error'))))
         else:
-            self.socket.setsockopt(socket.SOL_SOCKET,
-                                   SO_UPDATE_CONNECT_CONTEXT,
-                                   struct.pack('I', self.socket.fileno()))
+            self.socket.setsockopt(
+                socket.SOL_SOCKET, SO_UPDATE_CONNECT_CONTEXT,
+                struct.pack('P', self.socket.fileno()))
             self.protocol = self.connector.buildProtocol(self.getPeer())
             self.connected = True
-            self.logstr = self.protocol.__class__.__name__+",client"
+            logPrefix = self._getLogPrefix(self.protocol)
+            self.logstr = logPrefix + ",client"
             self.protocol.makeConnection(self)
             self.startReading()
 
@@ -389,6 +370,10 @@ class Server(Connection):
 
     I am a serverside network connection transport; a socket which came from an
     accept() on a server.
+
+    @ivar _tlsClientDefault: Always C{False}, indicating that this is a server
+        connection, and by default when TLS is negotiated this class will act as
+        a TLS server.
     """
 
     _tlsClientDefault = False
@@ -406,8 +391,8 @@ class Server(Connection):
         self.serverAddr = serverAddr
         self.clientAddr = clientAddr
         self.sessionno = sessionno
-        self.logstr = "%s,%s,%s" % (self.protocol.__class__.__name__,
-                                    sessionno, self.clientAddr.host)
+        logPrefix = self._getLogPrefix(self.protocol)
+        self.logstr = "%s,%s,%s" % (logPrefix, sessionno, self.clientAddr.host)
         self.repstr = "<%s #%s on %s>" % (self.protocol.__class__.__name__,
                                           self.sessionno, self.serverAddr.port)
         self.connected = True
@@ -447,7 +432,7 @@ class Connector(TCPConnector):
 
 
 
-class Port(_SocketCloser):
+class Port(_SocketCloser, _LogOwner):
     implements(interfaces.IListeningPort)
 
     connected = False
@@ -455,13 +440,18 @@ class Port(_SocketCloser):
     disconnecting = False
     addressFamily = socket.AF_INET
     socketType = socket.SOCK_STREAM
-
+    _addressType = address.IPv4Address
     sessionno = 0
 
     # Actual port number being listened on, only set to a non-None
     # value when we are actually listening.
     _realPortNumber = None
 
+    # A string describing the connections which will be created by this port.
+    # Normally this is C{"TCP"}, since this is a TCP port, but when the TLS
+    # implementation re-uses this class it overrides the value with C{"TLS"}.
+    # Only used for logging.
+    _type = 'TCP'
 
     def __init__(self, port, factory, backlog=50, interface='', reactor=None):
         self.port = port
@@ -469,6 +459,9 @@ class Port(_SocketCloser):
         self.backlog = backlog
         self.interface = interface
         self.reactor = reactor
+        if isIPv6Address(interface):
+            self.addressFamily = socket.AF_INET6
+            self._addressType = address.IPv6Address
 
 
     def __repr__(self):
@@ -486,7 +479,11 @@ class Port(_SocketCloser):
             skt = self.reactor.createSocket(self.addressFamily,
                                             self.socketType)
             # TODO: resolve self.interface if necessary
-            skt.bind((self.interface, self.port))
+            if self.addressFamily == socket.AF_INET6:
+                addr = socket.getaddrinfo(self.interface, self.port)[0][4]
+            else:
+                addr = (self.interface, self.port)
+            skt.bind(addr)
         except socket.error, le:
             raise error.CannotListenError, (self.interface, self.port, le)
 
@@ -496,7 +493,7 @@ class Port(_SocketCloser):
         # reflect what the OS actually assigned us.
         self._realPortNumber = skt.getsockname()[1]
 
-        log.msg("%s starting on %s" % (self.factory.__class__,
+        log.msg("%s starting on %s" % (self._getLogPrefix(self.factory),
                                        self._realPortNumber))
 
         self.factory.doStart()
@@ -530,7 +527,7 @@ class Port(_SocketCloser):
         """
         Log message for closing port
         """
-        log.msg('(TCP Port %s Closed)' % (self._realPortNumber,))
+        log.msg('(%s Port %s Closed)' % (self._type, self._realPortNumber))
 
 
     def connectionLost(self, reason):
@@ -547,7 +544,7 @@ class Port(_SocketCloser):
         self.disconnected = True
         self.reactor.removeActiveHandle(self)
         self.connected = False
-        self._closeSocket()
+        self._closeSocket(True)
         del self.socket
         del self.getFileHandle
 
@@ -578,7 +575,8 @@ class Port(_SocketCloser):
 
         This indicates the server's address.
         """
-        return address.IPv4Address('TCP', *self.socket.getsockname())
+        host, port = self.socket.getsockname()[:2]
+        return self._addressType('TCP', host, port)
 
 
     def cbAccept(self, rc, bytes, evt):
@@ -598,22 +596,23 @@ class Port(_SocketCloser):
                     (errno.errorcode.get(rc, 'unknown error'), rc))
             return False
         else:
-            evt.newskt.setsockopt(socket.SOL_SOCKET, SO_UPDATE_ACCEPT_CONTEXT,
-                                  struct.pack('I', self.socket.fileno()))
+            evt.newskt.setsockopt(
+                socket.SOL_SOCKET, SO_UPDATE_ACCEPT_CONTEXT,
+                struct.pack('P', self.socket.fileno()))
             family, lAddr, rAddr = _iocp.get_accept_addrs(evt.newskt.fileno(),
                                                           evt.buff)
             assert family == self.addressFamily
 
             protocol = self.factory.buildProtocol(
-                address._ServerFactoryIPv4Address('TCP', rAddr[0], rAddr[1]))
+                self._addressType('TCP', rAddr[0], rAddr[1]))
             if protocol is None:
                 evt.newskt.close()
             else:
                 s = self.sessionno
                 self.sessionno = s+1
                 transport = Server(evt.newskt, protocol,
-                        address.IPv4Address('TCP', rAddr[0], rAddr[1]),
-                        address.IPv4Address('TCP', lAddr[0], lAddr[1]),
+                        self._addressType('TCP', rAddr[0], rAddr[1]),
+                        self._addressType('TCP', lAddr[0], lAddr[1]),
                         s, self.reactor)
                 protocol.makeConnection(transport)
             return True
