@@ -1,4 +1,4 @@
-# Copyright (c) 2001-2009 Twisted Matrix Laboratories.
+# Copyright (c) Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 """
@@ -7,9 +7,12 @@ Test cases for twisted.protocols package.
 
 import struct
 
+from zope.interface.verify import verifyObject
+
 from twisted.trial import unittest
 from twisted.protocols import basic, wire, portforward
-from twisted.internet import reactor, protocol, defer, task, error
+from twisted.internet import reactor, protocol, defer, task, error, address
+from twisted.internet.interfaces import IProtocolFactory, ILoggingContext
 from twisted.test import proto_helpers
 
 
@@ -112,6 +115,31 @@ class LineOnlyTester(basic.LineOnlyReceiver):
 
 
 
+class FactoryTests(unittest.TestCase):
+    """
+    Tests for L{protocol.Factory}.
+    """
+    def test_interfaces(self):
+        """
+        L{protocol.Factory} instances provide both L{IProtocolFactory} and
+        L{ILoggingContext}.
+        """
+        factory = protocol.Factory()
+        self.assertTrue(verifyObject(IProtocolFactory, factory))
+        self.assertTrue(verifyObject(ILoggingContext, factory))
+
+
+    def test_logPrefix(self):
+        """
+        L{protocol.Factory.logPrefix} returns the name of the factory class.
+        """
+        class SomeKindOfFactory(protocol.Factory):
+            pass
+
+        self.assertEqual("SomeKindOfFactory", SomeKindOfFactory().logPrefix())
+
+
+
 class WireTestCase(unittest.TestCase):
     """
     Test wire protocols.
@@ -129,7 +157,7 @@ class WireTestCase(unittest.TestCase):
         a.dataReceived("how")
         a.dataReceived("are")
         a.dataReceived("you")
-        self.assertEquals(t.value(), "helloworldhowareyou")
+        self.assertEqual(t.value(), "helloworldhowareyou")
 
 
     def test_who(self):
@@ -139,7 +167,7 @@ class WireTestCase(unittest.TestCase):
         t = proto_helpers.StringTransport()
         a = wire.Who()
         a.makeConnection(t)
-        self.assertEquals(t.value(), "root\r\n")
+        self.assertEqual(t.value(), "root\r\n")
 
 
     def test_QOTD(self):
@@ -149,7 +177,7 @@ class WireTestCase(unittest.TestCase):
         t = proto_helpers.StringTransport()
         a = wire.QOTD()
         a.makeConnection(t)
-        self.assertEquals(t.value(),
+        self.assertEqual(t.value(),
                           "An apple a day keeps the doctor away.\r\n")
 
 
@@ -207,7 +235,7 @@ a'''
             for i in range(len(self.buffer)/packet_size + 1):
                 s = self.buffer[i*packet_size:(i+1)*packet_size]
                 a.dataReceived(s)
-            self.failUnlessEqual(self.output, a.received)
+            self.assertEqual(self.output, a.received)
 
 
     pause_buf = 'twiddle1\ntwiddle2\npause\ntwiddle3\n'
@@ -229,9 +257,9 @@ a'''
             for i in range(len(self.pause_buf)/packet_size + 1):
                 s = self.pause_buf[i*packet_size:(i+1)*packet_size]
                 a.dataReceived(s)
-            self.assertEquals(self.pause_output1, a.received)
+            self.assertEqual(self.pause_output1, a.received)
             clock.advance(0)
-            self.assertEquals(self.pause_output2, a.received)
+            self.assertEqual(self.pause_output2, a.received)
 
     rawpause_buf = 'twiddle1\ntwiddle2\nlen 5\nrawpause\n12345twiddle3\n'
 
@@ -252,9 +280,9 @@ a'''
             for i in range(len(self.rawpause_buf)/packet_size + 1):
                 s = self.rawpause_buf[i*packet_size:(i+1)*packet_size]
                 a.dataReceived(s)
-            self.assertEquals(self.rawpause_output1, a.received)
+            self.assertEqual(self.rawpause_output1, a.received)
             clock.advance(0)
-            self.assertEquals(self.rawpause_output2, a.received)
+            self.assertEqual(self.rawpause_output2, a.received)
 
     stop_buf = 'twiddle1\ntwiddle2\nstop\nmore\nstuff\n'
 
@@ -272,7 +300,7 @@ a'''
             for i in range(len(self.stop_buf)/packet_size + 1):
                 s = self.stop_buf[i*packet_size:(i+1)*packet_size]
                 a.dataReceived(s)
-            self.assertEquals(self.stop_output, a.received)
+            self.assertEqual(self.stop_output, a.received)
 
 
     def test_lineReceiverAsProducer(self):
@@ -283,7 +311,7 @@ a'''
         t = proto_helpers.StringIOWithoutClosing()
         a.makeConnection(protocol.FileWrapper(t))
         a.dataReceived('produce\nhello world\nunproduce\ngoodbye\n')
-        self.assertEquals(a.received,
+        self.assertEqual(a.received,
                           ['produce', 'hello world', 'unproduce', 'goodbye'])
 
 
@@ -329,7 +357,7 @@ class LineOnlyReceiverTestCase(unittest.TestCase):
         a.makeConnection(t)
         for c in self.buffer:
             a.dataReceived(c)
-        self.assertEquals(a.received, self.buffer.split('\n')[:-1])
+        self.assertEqual(a.received, self.buffer.split('\n')[:-1])
 
 
     def test_lineTooLong(self):
@@ -431,7 +459,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
                 s = out[i*packet_size:(i+1)*packet_size]
                 if s:
                     a.dataReceived(s)
-            self.assertEquals(a.received, self.strings)
+            self.assertEqual(a.received, self.strings)
 
 
     def test_sendNonStrings(self):
@@ -451,8 +479,8 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
             t.clear()
             length = out[:out.find(":")]
             data = out[out.find(":") + 1:-1] #[:-1] to ignore the trailing ","
-            self.assertEquals(int(length), len(str(s)))
-            self.assertEquals(data, str(s))
+            self.assertEqual(int(length), len(str(s)))
+            self.assertEqual(data, str(s))
 
         warnings = self.flushWarnings(
             offendingFunctions=[self.test_sendNonStrings])
@@ -471,7 +499,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         Empty netstrings (with length '0') can be received.
         """
         self.netstringReceiver.dataReceived("0:,")
-        self.assertEquals(self.netstringReceiver.received, [""])
+        self.assertEqual(self.netstringReceiver.received, [""])
 
 
     def test_receiveOneCharacter(self):
@@ -479,7 +507,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         One-character netstrings can be received.
         """
         self.netstringReceiver.dataReceived("1:a,")
-        self.assertEquals(self.netstringReceiver.received, ["a"])
+        self.assertEqual(self.netstringReceiver.received, ["a"])
 
 
     def test_receiveTwoCharacters(self):
@@ -487,7 +515,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         Two-character netstrings can be received.
         """
         self.netstringReceiver.dataReceived("2:ab,")
-        self.assertEquals(self.netstringReceiver.received, ["ab"])
+        self.assertEqual(self.netstringReceiver.received, ["ab"])
 
 
     def test_receiveNestedNetstring(self):
@@ -497,7 +525,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         characters appearing inside the data portion of the netstring.
         """
         self.netstringReceiver.dataReceived("4:1:a,,")
-        self.assertEquals(self.netstringReceiver.received, ["1:a,"])
+        self.assertEqual(self.netstringReceiver.received, ["1:a,"])
 
 
     def test_moreDataThanSpecified(self):
@@ -558,7 +586,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         """
         self.netstringReceiver.dataReceived("4:aa")
         self.netstringReceiver.dataReceived("aa,")
-        self.assertEquals(self.netstringReceiver.received, ["aaaa"])
+        self.assertEqual(self.netstringReceiver.received, ["aaaa"])
         self.assertTrue(self.netstringReceiver._payloadComplete())
 
 
@@ -569,7 +597,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         """
         for part in ["1", "0:01234", "56789", ","]:
             self.netstringReceiver.dataReceived(part)
-        self.assertEquals(self.netstringReceiver.received, ["0123456789"])
+        self.assertEqual(self.netstringReceiver.received, ["0123456789"])
 
 
     def test_receiveNetstringPortions_3(self):
@@ -578,7 +606,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         """
         for part in "2:ab,":
             self.netstringReceiver.dataReceived(part)
-        self.assertEquals(self.netstringReceiver.received, ["ab"])
+        self.assertEqual(self.netstringReceiver.received, ["ab"])
 
 
     def test_receiveTwoNetstrings(self):
@@ -589,9 +617,9 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         """
         self.netstringReceiver.dataReceived("1:a,1")
         self.assertTrue(self.netstringReceiver._payloadComplete())
-        self.assertEquals(self.netstringReceiver.received, ["a"])
+        self.assertEqual(self.netstringReceiver.received, ["a"])
         self.netstringReceiver.dataReceived(":b,")
-        self.assertEquals(self.netstringReceiver.received, ["a", "b"])
+        self.assertEqual(self.netstringReceiver.received, ["a", "b"])
 
 
     def test_maxReceiveLimit(self):
@@ -612,7 +640,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         """
         self.netstringReceiver._remainingData = "12:"
         self.netstringReceiver._consumeLength()
-        self.assertEquals(self.netstringReceiver._expectedPayloadSize, 13)
+        self.assertEqual(self.netstringReceiver._expectedPayloadSize, 13)
 
 
     def test_consumeLengthBorderCase1(self):
@@ -623,7 +651,7 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         self.netstringReceiver._remainingData = "12:"
         self.netstringReceiver.MAX_LENGTH = 12
         self.netstringReceiver._consumeLength()
-        self.assertEquals(self.netstringReceiver._expectedPayloadSize, 13)
+        self.assertEqual(self.netstringReceiver._expectedPayloadSize, 13)
 
 
     def test_consumeLengthBorderCase2(self):
@@ -659,22 +687,22 @@ class NetstringReceiverTestCase(unittest.TestCase, LPTestCaseMixin):
         warnings = self.flushWarnings(
             offendingFunctions=[self.test_deprecatedModuleAttributes])
 
-        self.assertEquals(len(warnings), 4)
+        self.assertEqual(len(warnings), 4)
         for warning in warnings:
-            self.assertEquals(warning['category'], DeprecationWarning)
-        self.assertEquals(
+            self.assertEqual(warning['category'], DeprecationWarning)
+        self.assertEqual(
             warnings[0]['message'],
             ("twisted.protocols.basic.LENGTH was deprecated in Twisted 10.2.0: "
              "NetstringReceiver parser state is private."))
-        self.assertEquals(
+        self.assertEqual(
             warnings[1]['message'],
             ("twisted.protocols.basic.DATA was deprecated in Twisted 10.2.0: "
              "NetstringReceiver parser state is private."))
-        self.assertEquals(
+        self.assertEqual(
             warnings[2]['message'],
             ("twisted.protocols.basic.COMMA was deprecated in Twisted 10.2.0: "
              "NetstringReceiver parser state is private."))
-        self.assertEquals(
+        self.assertEqual(
             warnings[3]['message'],
             ("twisted.protocols.basic.NUMBER was deprecated in Twisted 10.2.0: "
              "NetstringReceiver parser state is private."))
@@ -699,7 +727,7 @@ class IntNTestCaseMixin(LPTestCaseMixin):
         for s in self.strings:
             for c in struct.pack(r.structFormat,len(s)) + s:
                 r.dataReceived(c)
-        self.assertEquals(r.received, self.strings)
+        self.assertEqual(r.received, self.strings)
 
 
     def test_partial(self):
@@ -710,7 +738,7 @@ class IntNTestCaseMixin(LPTestCaseMixin):
             r = self.getProtocol()
             for c in s:
                 r.dataReceived(c)
-            self.assertEquals(r.received, [])
+            self.assertEqual(r.received, [])
 
 
     def test_send(self):
@@ -719,7 +747,7 @@ class IntNTestCaseMixin(LPTestCaseMixin):
         """
         r = self.getProtocol()
         r.sendString("b" * 16)
-        self.assertEquals(r.transport.value(),
+        self.assertEqual(r.transport.value(),
             struct.pack(r.structFormat, 16) + "b" * 16)
 
 
@@ -751,6 +779,111 @@ class IntNTestCaseMixin(LPTestCaseMixin):
 
 
 
+class RecvdAttributeMixin(object):
+    """
+    Mixin defining tests for string receiving protocols with a C{recvd}
+    attribute which should be settable by application code, to be combined with
+    L{IntNTestCaseMixin} on a L{TestCase} subclass
+    """
+
+    def makeMessage(self, protocol, data):
+        """
+        Return C{data} prefixed with message length in C{protocol.structFormat}
+        form.
+        """
+        return struct.pack(protocol.structFormat, len(data)) + data
+
+
+    def test_recvdContainsRemainingData(self):
+        """
+        In stringReceived, recvd contains the remaining data that was passed to
+        dataReceived that was not part of the current message.
+        """
+        result = []
+        r = self.getProtocol()
+        def stringReceived(receivedString):
+            result.append(r.recvd)
+        r.stringReceived = stringReceived
+        completeMessage = (struct.pack(r.structFormat, 5) + ('a' * 5))
+        incompleteMessage = (struct.pack(r.structFormat, 5) + ('b' * 4))
+        # Receive a complete message, followed by an incomplete one
+        r.dataReceived(completeMessage + incompleteMessage)
+        self.assertEquals(result, [incompleteMessage])
+
+
+    def test_recvdChanged(self):
+        """
+        In stringReceived, if recvd is changed, messages should be parsed from
+        it rather than the input to dataReceived.
+        """
+        r = self.getProtocol()
+        result = []
+        payloadC = 'c' * 5
+        messageC = self.makeMessage(r, payloadC)
+        def stringReceived(receivedString):
+            if not result:
+                r.recvd = messageC
+            result.append(receivedString)
+        r.stringReceived = stringReceived
+        payloadA = 'a' * 5
+        payloadB = 'b' * 5
+        messageA = self.makeMessage(r, payloadA)
+        messageB = self.makeMessage(r, payloadB)
+        r.dataReceived(messageA + messageB)
+        self.assertEquals(result, [payloadA, payloadC])
+
+
+    def test_switching(self):
+        """
+        Data already parsed by L{IntNStringReceiver.dataReceived} is not
+        reparsed if C{stringReceived} consumes some of the
+        L{IntNStringReceiver.recvd} buffer.
+        """
+        proto = self.getProtocol()
+        mix = []
+        SWITCH = "\x00\x00\x00\x00"
+        for s in self.strings:
+            mix.append(self.makeMessage(proto, s))
+            mix.append(SWITCH)
+
+        result = []
+        def stringReceived(receivedString):
+            result.append(receivedString)
+            proto.recvd = proto.recvd[len(SWITCH):]
+
+        proto.stringReceived = stringReceived
+        proto.dataReceived("".join(mix))
+        # Just another byte, to trigger processing of anything that might have
+        # been left in the buffer (should be nothing).
+        proto.dataReceived("\x01")
+        self.assertEqual(result, self.strings)
+        # And verify that another way
+        self.assertEqual(proto.recvd, "\x01")
+
+
+    def test_recvdInLengthLimitExceeded(self):
+        """
+        The L{IntNStringReceiver.recvd} buffer contains all data not yet
+        processed by L{IntNStringReceiver.dataReceived} if the
+        C{lengthLimitExceeded} event occurs.
+        """
+        proto = self.getProtocol()
+        DATA = "too long"
+        proto.MAX_LENGTH = len(DATA) - 1
+        message = self.makeMessage(proto, DATA)
+
+        result = []
+        def lengthLimitExceeded(length):
+            result.append(length)
+            result.append(proto.recvd)
+
+        proto.lengthLimitExceeded = lengthLimitExceeded
+        proto.dataReceived(message)
+        self.assertEqual(result[0], len(DATA))
+        self.assertEqual(result[1], message)
+
+
+
 class TestInt32(TestMixin, basic.Int32StringReceiver):
     """
     A L{basic.Int32StringReceiver} storing received strings in an array.
@@ -760,7 +893,7 @@ class TestInt32(TestMixin, basic.Int32StringReceiver):
 
 
 
-class Int32TestCase(unittest.TestCase, IntNTestCaseMixin):
+class Int32TestCase(unittest.TestCase, IntNTestCaseMixin, RecvdAttributeMixin):
     """
     Test case for int32-prefixed protocol
     """
@@ -775,9 +908,9 @@ class Int32TestCase(unittest.TestCase, IntNTestCaseMixin):
         """
         r = self.getProtocol()
         r.sendString("foo")
-        self.assertEquals(r.transport.value(), "\x00\x00\x00\x03foo")
+        self.assertEqual(r.transport.value(), "\x00\x00\x00\x03foo")
         r.dataReceived("\x00\x00\x00\x04ubar")
-        self.assertEquals(r.received, ["ubar"])
+        self.assertEqual(r.received, ["ubar"])
 
 
 
@@ -790,7 +923,7 @@ class TestInt16(TestMixin, basic.Int16StringReceiver):
 
 
 
-class Int16TestCase(unittest.TestCase, IntNTestCaseMixin):
+class Int16TestCase(unittest.TestCase, IntNTestCaseMixin, RecvdAttributeMixin):
     """
     Test case for int16-prefixed protocol
     """
@@ -805,9 +938,9 @@ class Int16TestCase(unittest.TestCase, IntNTestCaseMixin):
         """
         r = self.getProtocol()
         r.sendString("foo")
-        self.assertEquals(r.transport.value(), "\x00\x03foo")
+        self.assertEqual(r.transport.value(), "\x00\x03foo")
         r.dataReceived("\x00\x04ubar")
-        self.assertEquals(r.received, ["ubar"])
+        self.assertEqual(r.received, ["ubar"])
 
 
     def test_tooLongSend(self):
@@ -820,6 +953,22 @@ class Int16TestCase(unittest.TestCase, IntNTestCaseMixin):
 
 
 
+class NewStyleTestInt16(TestInt16, object):
+    """
+    A new-style class version of TestInt16
+    """
+
+
+
+class NewStyleInt16TestCase(Int16TestCase):
+    """
+    This test case verifies that IntNStringReceiver still works when inherited
+    by a new-style class.
+    """
+    protocol = NewStyleTestInt16
+
+
+
 class TestInt8(TestMixin, basic.Int8StringReceiver):
     """
     A L{basic.Int8StringReceiver} storing received strings in an array.
@@ -829,7 +978,7 @@ class TestInt8(TestMixin, basic.Int8StringReceiver):
 
 
 
-class Int8TestCase(unittest.TestCase, IntNTestCaseMixin):
+class Int8TestCase(unittest.TestCase, IntNTestCaseMixin, RecvdAttributeMixin):
     """
     Test case for int8-prefixed protocol
     """
@@ -845,9 +994,9 @@ class Int8TestCase(unittest.TestCase, IntNTestCaseMixin):
         """
         r = self.getProtocol()
         r.sendString("foo")
-        self.assertEquals(r.transport.value(), "\x03foo")
+        self.assertEqual(r.transport.value(), "\x03foo")
         r.dataReceived("\x04ubar")
-        self.assertEquals(r.received, ["ubar"])
+        self.assertEqual(r.received, ["ubar"])
 
 
     def test_tooLongSend(self):
@@ -907,7 +1056,7 @@ class ProducerTestCase(unittest.TestCase):
 
         p.dataReceived('world\r\n')
 
-        self.assertEquals(t.data, ['hello, world'])
+        self.assertEqual(t.data, ['hello, world'])
         self.failUnless(t.paused)
         self.failUnless(p.paused)
 
@@ -918,26 +1067,26 @@ class ProducerTestCase(unittest.TestCase):
 
         p.dataReceived('hello\r\nworld\r\n')
 
-        self.assertEquals(t.data, ['hello, world', 'hello'])
+        self.assertEqual(t.data, ['hello, world', 'hello'])
         self.failUnless(t.paused)
         self.failUnless(p.paused)
 
         p.resumeProducing()
         p.dataReceived('goodbye\r\n')
 
-        self.assertEquals(t.data, ['hello, world', 'hello', 'world'])
+        self.assertEqual(t.data, ['hello, world', 'hello', 'world'])
         self.failUnless(t.paused)
         self.failUnless(p.paused)
 
         p.resumeProducing()
 
-        self.assertEquals(t.data, ['hello, world', 'hello', 'world', 'goodbye'])
+        self.assertEqual(t.data, ['hello, world', 'hello', 'world', 'goodbye'])
         self.failUnless(t.paused)
         self.failUnless(p.paused)
 
         p.resumeProducing()
 
-        self.assertEquals(t.data, ['hello, world', 'hello', 'world', 'goodbye'])
+        self.assertEqual(t.data, ['hello, world', 'hello', 'world', 'goodbye'])
         self.failIf(t.paused)
         self.failIf(p.paused)
 
@@ -1042,7 +1191,7 @@ class Portforwarding(unittest.TestCase):
         def testDataReceived(data):
             received.extend(data)
             if len(received) >= nBytes:
-                self.assertEquals(''.join(received), 'x' * nBytes)
+                self.assertEqual(''.join(received), 'x' * nBytes)
                 d.callback(None)
 
         self.clientProtocol.dataReceived = testDataReceived
@@ -1061,6 +1210,42 @@ class Portforwarding(unittest.TestCase):
         return d
 
 
+    def test_registerProducers(self):
+        """
+        The proxy client registers itself as a producer of the proxy server and
+        vice versa.
+        """
+        # create a ProxyServer instance
+        addr = address.IPv4Address('TCP', '127.0.0.1', 0)
+        server = portforward.ProxyFactory('127.0.0.1', 0).buildProtocol(addr)
+
+        # set the reactor for this test
+        reactor = proto_helpers.MemoryReactor()
+        server.reactor = reactor
+
+        # make the connection
+        serverTransport = proto_helpers.StringTransport()
+        server.makeConnection(serverTransport)
+
+        # check that the ProxyClientFactory is connecting to the backend
+        self.assertEqual(len(reactor.tcpClients), 1)
+        # get the factory instance and check it's the one we expect
+        host, port, clientFactory, timeout, _ = reactor.tcpClients[0]
+        self.assertIsInstance(clientFactory, portforward.ProxyClientFactory)
+
+        # Connect it
+        client = clientFactory.buildProtocol(addr)
+        clientTransport = proto_helpers.StringTransport()
+        client.makeConnection(clientTransport)
+
+        # check that the producers are registered
+        self.assertIdentical(clientTransport.producer, serverTransport)
+        self.assertIdentical(serverTransport.producer, clientTransport)
+        # check the streaming attribute in both transports
+        self.assertTrue(clientTransport.streaming)
+        self.assertTrue(serverTransport.streaming)
+
+
 
 class StringTransportTestCase(unittest.TestCase):
     """
